# -*- coding: utf-8 -*-
from __future__ import unicode_literals
import re
import os
import sys
import time
from django.db import connection


DEBUG=False
if __name__ == "__main__":
    import django
    web_path = os.path.realpath(os.path.join(os.path.dirname(__file__), '../../'))
    sys.path.append(web_path)
    os.environ.setdefault("DJANGO_SETTINGS_MODULE", 'operation.settings')
    django.setup()
    DEBUG=True

from django.db.models import Q, Count, Sum
from app.core.models import (
    Mailbox, MailboxUser, MailboxSize, DomainAttr,
    Domain, CoreMonitor, CoreAlias, Department, DepartmentMember, VisitLog, AuthLog )
from app.rpt.models import  MailLog, LogReport
from django.utils.translation import ugettext_lazy as _

def add_condition( cond, q ):
    if not cond:
        return q
    return cond & q

def get_daystart(delta=0):
    now = int(time.time())
    midnight = now - (now % 86400) + time.timezone
    premidnight = midnight- 86400*delta
    return time.strftime('%Y-%m-%d 00:00:00',time.localtime(premidnight))

def get_save_days():
    save_days = 180
    obj = DomainAttr.objects.filter(domain_id=0, type='system', item='sw_mail_log_save_day').first()
    if obj:
        save_days = int(obj.value)
    return save_days

def get_date_offset(domain_id):
    save_days = get_save_days()
    l = []
    for v in xrange(-1,save_days):
        if v == -1:
            l.append( (-1,_(u"--")) )
            continue
        date_start = get_daystart(int(v))
        l.append( (v, '%s'%date_start ) )
    return tuple(l)

def get_department_list():
    l = [(u"0",_(u"--"))]
    for obj in Department.objects.all():
        l.append( (obj.id,obj.title) )
    return tuple(l)

def get_day(delta=0):
    if delta<0:
        return time.strftime('%Y-%m-%d %H:%M:%S')
    now = int(time.time())
    midnight = now - (now % 86400) + time.timezone
    premidnight = midnight- 86400*delta
    return time.strftime('%Y-%m-%d',time.localtime(premidnight))

def add_force_index(sql, force_index):
    # 正则表达式匹配 FROM 后面跟随的表名，并捕获表名
    pattern = r"FROM\s+`(?P<table_name>[\w\d_]+)`"

    # 定义替换函数，使用捕获的表名加上 FORCE INDEX
    def replacer(match):
        table_name = match.group("table_name")
        return "FROM `{}` FORCE INDEX (`{}`)".format(table_name, force_index)

    # 使用正则表达式进行替换
    return re.sub(pattern, replacer, sql)

def get_count_with_optimized_index(lists, condition, force_index):

    query = lists.filter(condition)
    sql, params = query.query.get_compiler(connection=connection).as_sql()

    # 修改SQL以强制使用索引并获取计数
    sql = add_force_index(sql, force_index)

    sql_count = "SELECT COUNT(*) FROM ({}) as subquery".format(sql)

    with connection.cursor() as cursor:
        cursor.execute(sql_count, params)
        count = cursor.fetchone()[0]

    return count

def get_mail_stat_data(domain_id, mailbox_id, t):
    save_days = get_save_days()
    save_start = get_daystart(save_days-1)

    save_start_30 = get_daystart(29)

    def get_stat(lists,condition,type,key,force_index=''):
        data = LogReport.get_cache(domain_id,mailbox_id,type,key)
        if not data:
            if force_index:
                total = get_count_with_optimized_index(lists, condition, force_index)
            else:
                total = lists.filter( condition ).count()
            data = {"total":total}
            LogReport.save_cache(domain_id,mailbox_id,type,key,data)
        total = int(data["total"])
        return total
    def check_sql_domain(sql):
        if domain_id:
            sql += " AND `domain_id`={}".format(domain_id)
        sql = sql.replace('`', '')
        return sql
    def count_maillog_today(t,today,save_start):
        sql_total = "select count(1) from core_mail_log force index (date_range) where `type`='{}' and `recv_time_date`>='{}'".format(t, save_start)
        sql_total = check_sql_domain(sql_total)
        stat_total = MailLog.objects.extra(select={'count':sql_total}).first()
        size_total = 0 if not stat_total else stat_total.count

        sql_today = "select count(1) from core_mail_log force index (date_range) where `type`='{}' and `recv_time_date`>='{}'".format( t, today )
        sql_today = check_sql_domain(sql_today)
        stat_today = MailLog.objects.extra(select={'count':sql_today}).first()
        size_today = 0 if not stat_today else stat_today.count
        return size_total, size_today
    def count_maillog_today2(t,today,save_start):
        sql_total = "select count(1) from core_mail_log force index (date_status_range) where `status`='{}' and `recv_time_date`>='{}'".format(t, save_start)
        sql_total = check_sql_domain(sql_total)
        stat_total = MailLog.objects.extra(select={'count':sql_total}).first()
        size_total = 0 if not stat_total else stat_total.count

        sql_today = "select count(1) from core_mail_log  force index (date_status_range) where `status`='{}' and `recv_time_date`>='{}'".format( t, today )
        sql_today = check_sql_domain(sql_today)
        stat_today = MailLog.objects.extra(select={'count':sql_today}).first()
        size_today = 0 if not stat_today else stat_today.count
        return size_total, size_today
    def count_authlog_today(t,today,save_start):
        sql_total = "select count(1) from core_auth_log  force index (date_range) where `type`='{}' and `date`>='{}'".format(t, save_start)
        sql_total = check_sql_domain(sql_total)
        stat_total = MailLog.objects.extra(select={'count':sql_total}).first()
        size_total = 0 if not stat_total else stat_total.count

        sql_today = "select count(1) from core_auth_log force index (date_range) where `type`='{}' and `date`>='{}'".format( t, today )
        sql_today = check_sql_domain(sql_today)
        stat_today = MailLog.objects.extra(select={'count':sql_today}).first()
        size_today = 0 if not stat_today else stat_today.count
        return size_total, size_today

    condition = None
    if domain_id:
        condition = Q(domain_id=domain_id)
    today = get_day()
    day_1 = get_day(1)
    day_2 = get_day(2)
    day_3 = get_day(3)
    day_4 = get_day(4)
    day_5 = get_day(5)
    day_6 = get_day(6)
    lists = MailLog.objects.all()
    if t == "smtp_in":
        size_total_30, _ = count_maillog_today('in', today, save_start_30)
        size_total, size_today = count_maillog_today('in', today, save_start)

        condition = add_condition(condition,Q(type='in'))
        lists = MailLog.objects.all()
        size_6 = get_stat( lists, condition & (Q(recv_time_date__gte=day_6) & Q(recv_time_date__lt=day_5)),t,day_6, force_index='date_range')
        size_5 = get_stat( lists, condition & (Q(recv_time_date__gte=day_5) & Q(recv_time_date__lt=day_4)),t,day_5, force_index='date_range')
        size_4 = get_stat( lists, condition & (Q(recv_time_date__gte=day_4) & Q(recv_time_date__lt=day_3)),t,day_4, force_index='date_range')
        size_3 = get_stat( lists, condition & (Q(recv_time_date__gte=day_3) & Q(recv_time_date__lt=day_2)),t,day_3, force_index='date_range')
        size_2 = get_stat( lists, condition & (Q(recv_time_date__gte=day_2) & Q(recv_time_date__lt=day_1)),t,day_2, force_index='date_range')
        size_1 = get_stat( lists, condition & (Q(recv_time_date__gte=day_1) & Q(recv_time_date__lt=today)),t,day_1, force_index='date_range')
        size_week = size_1 + size_2 + size_3 + size_4 + size_5 + size_6 + size_today
    elif t == "smtp_out":
        size_total_30, _ = count_maillog_today('out', today, save_start_30)
        size_total, size_today = count_maillog_today('out', today, save_start)

        condition = add_condition(condition,Q(type='out'))
        lists = MailLog.objects.all()
        size_6 = get_stat( lists, condition & (Q(recv_time_date__gte=day_6) & Q(recv_time_date__lt=day_5)),t,day_6, force_index='date_range')
        size_5 = get_stat( lists, condition & (Q(recv_time_date__gte=day_5) & Q(recv_time_date__lt=day_4)),t,day_5, force_index='date_range')
        size_4 = get_stat( lists, condition & (Q(recv_time_date__gte=day_4) & Q(recv_time_date__lt=day_3)),t,day_4, force_index='date_range')
        size_3 = get_stat( lists, condition & (Q(recv_time_date__gte=day_3) & Q(recv_time_date__lt=day_2)),t,day_3, force_index='date_range')
        size_2 = get_stat( lists, condition & (Q(recv_time_date__gte=day_2) & Q(recv_time_date__lt=day_1)),t,day_2, force_index='date_range')
        size_1 = get_stat( lists, condition & (Q(recv_time_date__gte=day_1) & Q(recv_time_date__lt=today)),t,day_1, force_index='date_range')
        size_week = size_1 + size_2 + size_3 + size_4 + size_5 + size_6 + size_today
    elif t == "pop3_session":
        size_total_30, _ = count_authlog_today('pop3', today, save_start_30)
        size_total, size_today = count_authlog_today('pop3', today, save_start)

        condition = add_condition(condition,Q(type='pop3'))
        lists = AuthLog.objects.all()
        size_6 = get_stat( lists, condition & (Q(date__gte=day_6) & Q(date__lt=day_5)),t,day_6, force_index='date_range')
        size_5 = get_stat( lists, condition & (Q(date__gte=day_5) & Q(date__lt=day_4)),t,day_5, force_index='date_range')
        size_4 = get_stat( lists, condition & (Q(date__gte=day_4) & Q(date__lt=day_3)),t,day_4, force_index='date_range')
        size_3 = get_stat( lists, condition & (Q(date__gte=day_3) & Q(date__lt=day_2)),t,day_3, force_index='date_range')
        size_2 = get_stat( lists, condition & (Q(date__gte=day_2) & Q(date__lt=day_1)),t,day_2, force_index='date_range')
        size_1 = get_stat( lists, condition & (Q(date__gte=day_1) & Q(date__lt=today)),t,day_1, force_index='date_range')
        size_week = size_1 + size_2 + size_3 + size_4 + size_5 + size_6 + size_today
    elif t == "imap_session":
        size_total_30, _ = count_authlog_today('imap', today, save_start_30)
        size_total, size_today = count_authlog_today('imap', today, save_start)

        condition = add_condition(condition,Q(type='imap'))
        lists = AuthLog.objects.all()
        size_6 = get_stat( lists, condition & (Q(date__gte=day_6) & Q(date__lt=day_5)),t,day_6, force_index='date_range')
        size_5 = get_stat( lists, condition & (Q(date__gte=day_5) & Q(date__lt=day_4)),t,day_5, force_index='date_range')
        size_4 = get_stat( lists, condition & (Q(date__gte=day_4) & Q(date__lt=day_3)),t,day_4, force_index='date_range')
        size_3 = get_stat( lists, condition & (Q(date__gte=day_3) & Q(date__lt=day_2)),t,day_3, force_index='date_range')
        size_2 = get_stat( lists, condition & (Q(date__gte=day_2) & Q(date__lt=day_1)),t,day_2, force_index='date_range')
        size_1 = get_stat( lists, condition & (Q(date__gte=day_1) & Q(date__lt=today)),t,day_1, force_index='date_range')
        size_week = size_1 + size_2 + size_3 + size_4 + size_5 + size_6 + size_today
    elif t == "spam_receive":
        size_total_30, _ = count_maillog_today2('spam-flag', today, save_start_30)
        size_total, size_today = count_maillog_today2('spam-flag', today, save_start)

        condition = add_condition(condition,Q(status='spam-flag'))
        lists = MailLog.objects.all()
        size_6 = get_stat( lists, condition & (Q(recv_time_date__gte=day_6) & Q(recv_time_date__lt=day_5)),t,day_6, force_index='date_range')
        size_5 = get_stat( lists, condition & (Q(recv_time_date__gte=day_5) & Q(recv_time_date__lt=day_4)),t,day_5, force_index='date_range')
        size_4 = get_stat( lists, condition & (Q(recv_time_date__gte=day_4) & Q(recv_time_date__lt=day_3)),t,day_4, force_index='date_range')
        size_3 = get_stat( lists, condition & (Q(recv_time_date__gte=day_3) & Q(recv_time_date__lt=day_2)),t,day_3, force_index='date_range')
        size_2 = get_stat( lists, condition & (Q(recv_time_date__gte=day_2) & Q(recv_time_date__lt=day_1)),t,day_2, force_index='date_range')
        size_1 = get_stat( lists, condition & (Q(recv_time_date__gte=day_1) & Q(recv_time_date__lt=today)),t,day_1, force_index='date_range')
        size_week = size_1 + size_2 + size_3 + size_4 + size_5 + size_6 + size_today
    elif t == "spam_reject":
        size_total_30, _ = count_maillog_today2('spam', today, save_start_30)
        size_total, size_today = count_maillog_today2('spam', today, save_start)

        condition = add_condition(condition,Q(status='spam'))
        lists = MailLog.objects.all()
        size_6 = get_stat( lists, condition & (Q(recv_time_date__gte=day_6) & Q(recv_time_date__lt=day_5)),t,day_6)
        size_5 = get_stat( lists, condition & (Q(recv_time_date__gte=day_5) & Q(recv_time_date__lt=day_4)),t,day_5)
        size_4 = get_stat( lists, condition & (Q(recv_time_date__gte=day_4) & Q(recv_time_date__lt=day_3)),t,day_4)
        size_3 = get_stat( lists, condition & (Q(recv_time_date__gte=day_3) & Q(recv_time_date__lt=day_2)),t,day_3)
        size_2 = get_stat( lists, condition & (Q(recv_time_date__gte=day_2) & Q(recv_time_date__lt=day_1)),t,day_2)
        size_1 = get_stat( lists, condition & (Q(recv_time_date__gte=day_1) & Q(recv_time_date__lt=today)),t,day_1)
        size_week = size_1 + size_2 + size_3 + size_4 + size_5 + size_6 + size_today
    elif t == "spam_virus":
        size_total_30, _ = count_maillog_today2('virus', today, save_start_30)
        size_total, size_today = count_maillog_today2('virus', today, save_start)

        condition = add_condition(condition,Q(status='virus'))
        lists = MailLog.objects.all()
        size_6 = get_stat( lists, condition & (Q(recv_time_date__gte=day_6) & Q(recv_time_date__lt=day_5)),t,day_6)
        size_5 = get_stat( lists, condition & (Q(recv_time_date__gte=day_5) & Q(recv_time_date__lt=day_4)),t,day_5)
        size_4 = get_stat( lists, condition & (Q(recv_time_date__gte=day_4) & Q(recv_time_date__lt=day_3)),t,day_4)
        size_3 = get_stat( lists, condition & (Q(recv_time_date__gte=day_3) & Q(recv_time_date__lt=day_2)),t,day_3)
        size_2 = get_stat( lists, condition & (Q(recv_time_date__gte=day_2) & Q(recv_time_date__lt=day_1)),t,day_2)
        size_1 = get_stat( lists, condition & (Q(recv_time_date__gte=day_1) & Q(recv_time_date__lt=today)),t,day_1)
        size_week = size_1 + size_2 + size_3 + size_4 + size_5 + size_6 + size_today
    #end if
    data = {
            "stat_total_30" :   size_total_30,
            "stat_total"    :   size_total,
            "stat_week"     :   size_week,
            "stat_6"        :   size_6,
            "stat_5"        :   size_5,
            "stat_4"        :   size_4,
            "stat_3"        :   size_3,
            "stat_2"        :   size_2,
            "stat_1"        :   size_1,
            "stat_today"   :   size_today,
        }
    return data

def get_sys_date_change_logs():
    filepath = "/usr/local/u-mail/app/log/time_monitor.log"
    if not os.path.exists(filepath):
        return []
    with open(filepath, "rb") as fobj:
        res = fobj.read()
    lst = []
    for idx,line in enumerate(res.split('\n')):
        if not line.strip():
            continue
        lst.append( (idx,line) )
    lst.reverse()
    return lst

if __name__ == "__main__":
    msg = get_sys_date_change_logs()
    print msg
